function [config, fname,vnames] = guiepalg(config_i, fname_i, vnames_i)
%
% This is graphical user interface for EPALG - Evolutionary Programming ALGorithm
%

% check if figure already exists
h0 = findobj(0,'Tag','guiepalg');

if ( isempty(h0) ) % create figure and controls

set(0,'DefaultAxesFontSize',10,'DefaultTextFontSize',10,'DefaultUicontrolFontSize',10)
h0 = figure; 
%set(h0, 'WindowStyle','modal')
set(h0,'Units','normalized','Position',[0.05 0.050 0.90 0.85],'NumberTitle','off','Name','EPALG','MenuBar','none')
set(h0,'CloseRequestFcn','delete(gcf)')
set(h0,'WindowButtonDownFcn','epalghlp')
set(h0,'Tag','guiepalg')

% axes for results      
ha = axes('Position',[0.45 0.45 0.50 0.50]);
set(ha,'Tag','axepalg')

% frame for input data
uicontrol('Parent',h0, ...
          'Units','normalized', ...
          'Position',[0.02 0.02 0.35 0.96], ...
          'Style','frame');

% frame & edit box for explanation of parameters
uicontrol('Parent',h0, ...
          'Units','normalized', ...
          'Position',[0.45 0.02 0.51 0.35], ...
          'Style','frame');

uicontrol('Parent',h0, ...
          'Units','normalized', ...
          'Position',[0.46 0.03 0.49 0.33], ...
          'Style','edit', ...
          'Max',10, ...
          'Tag','help');

xl = 0.15;
yl = 0.90;
dx = 0.05;
dy = 0.05;
frdy = 0.8;
frdx = 2.0;
ih = 0;

% datatype
datatype = {'bit','unsigned_integer','real'};
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String',{'bit ';'int ';'real'}, ...
                   'Style','popupmenu', ...
                   'Tag','datatype', ...
                   'ToolTipString','Choose datatype that you will use', ... 
                   'UserData',datatype, ...
                   'Value',1);
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','datatype', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               

% integer_size
yl = yl;
xl = xl + frdx*dx;
integer_size = 8;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx dy], ...
                    'String', int2str(integer_size), ...
                    'Style','edit', ...
                    'Tag','integer_size', ...
                    'ToolTipString','Defines the number of bits that are used to store integer value.', ... 
                    'UserData', integer_size );
xl = xl - frdx*dx;

% generations
yl = yl - dy;
generations = 100;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx frdy*dy], ...
                    'String',int2str(generations), ...
                    'Style','edit', ...
                    'Tag','generations', ...
                    'ToolTipString','Number of generations', ...                     
                    'UserData', generations );
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx frdy*dy], ...
                   'String','generations', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               
                
% population
yl = yl - dy;
population = 20;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx frdy*dy], ...
                    'String',int2str(population), ...
                    'Style','edit', ...
                    'Tag','population', ...
                    'ToolTipString','Population size', ...                     
                    'UserData', population );
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx frdy*dy], ...
                   'String','population', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               
                
% length
yl = yl - dy;
length = 10;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx frdy*dy], ...
                    'String',int2str(length), ...
                    'Style','edit', ...
                    'Tag','length', ...
                    'ToolTipString','Number of genes in the chromosome.', ... 
                    'UserData', length );
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx frdy*dy], ...
                   'String','length', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               
                
% seed
yl = yl - dy;
seed = 0;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx frdy*dy], ...
                    'String',int2str(seed), ...
                    'Style','edit', ...
                    'Tag','seed', ...
                    'ToolTipString','Seed number for random number generator.', ...                     
                    'UserData', seed );
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx frdy*dy], ...
                   'String','seed', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               
                
% selection                
yl = yl - dy/frdy;
selection = {'roulette','tournament'};
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx dy], ...
                    'String',selection, ...
                    'Style','popupmenu', ...
                    'Tag','selection', ...
                    'ToolTipString','Selection type. For tournament selection enter the tournament size.', ... 
                    'Value',1, ...
                    'UserData', selection );
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','selection', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               

yl = yl;               
xl = xl + frdx*dx;
selection_tournament_size = 2;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx dy], ...
                    'String',int2str(selection_tournament_size), ...
                    'Style','edit', ...
                    'Tag','selection_tournament_size', ...
                    'ToolTipString','Tournament size (default is 2).', ... 
                    'UserData', selection_tournament_size );
xl = xl - frdx*dx;

% normalisation
yl = yl - dy/frdy;                
%normalisation = {'direct','reverse','invert','scale','reverse_scale','mean_scale', ... 
%                 'reverse_mean_scale','rank_linear','reverse_rank_linear','rank_geometric','reverse_rank_geometric'};
normalisation = {'direct','reverse','rank_linear','reverse_rank_linear'};

ih = ih + 1;         
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx dy], ...
                    'String',normalisation, ...
                    'Style','popupmenu', ...
                    'Value', 1 , ...
                    'Tag','normalisation', ...
                    'ToolTipString','Normalisation determines how to calculate fitness value from raw function value.', ...                     
                    'UserData', normalisation );
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','normalisation', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               
                
%yl = yl;
%xl = xl + frdx*dx;
%bias = 0;
%ih = ih + 1;
%hui(ih) = uicontrol('Parent',h0, ...
%                    'Units','normalized', ...
%                    'Position',[xl yl dx dy], ...
%                    'String', int2str(bias), ...
%                    'Style','edit', ...
%                    'Tag','bias', ...
%                    'ToolTipString','Bias value used for normalisation.', ...                     
%                    'UserData', bias );
%xl = xl - frdx*dx;

% replacement
yl = yl - dy/frdy;                
%replacement = {'uniform','ranked','tournament','crowding'};
replacement = {'uniform','ranked'};
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx dy], ...
                    'String',replacement, ...
                    'Style','popupmenu', ...
                    'Value', 1 , ...
                    'Tag','replacement', ...
                    'ToolTipString','Replacement type.', ...                     
                    'UserData', replacement ); 
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','replacement', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               

yl = yl;
xl = xl + frdx*dx;
replacement_rate = 1.0;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx dy], ...
                    'String', num2str(replacement_rate), ...
                    'Style','edit', ...
                    'Tag','replacement_rate', ...
                    'ToolTipString','Replacement rate determines relative part of the population that is to be replaced (0.0-1.0).', ...                     
                    'UserData', replacement_rate );
xl = xl - frdx*dx;

yl = yl - dy;                
elitism = {'on','off'};
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx dy], ...
                    'String',elitism, ...
                    'Style','popupmenu', ...
                    'Value', 1 , ...
                    'Tag','elitism', ...
                    'ToolTipString','If elitism is on, best chromosome is never replaced.', ...                     
                    'UserData', elitism ); 
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','elitism', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               

                
yl = yl - dy;                
%replacement_condition = {'unconditional','if_improved','annealed'};
replacement_condition = {'unconditional','if_improved'};
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx dy], ...
                    'String',replacement_condition, ...
                    'Style','popupmenu', ...
                    'Value', 1 , ...
                    'Tag','replacement_condition', ...
                    'ToolTipString','Replacement condition.', ...                     
                    'UserData', replacement_condition ); 
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','condition', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               
       
           
% mutation
yl = yl - dy/frdy;                
mutation = {'invert','reinit','step','uniform','gaussian'};
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx dy], ...
                    'String',mutation, ...
                    'Style','popupmenu', ...
                    'Value', 1 , ...
                    'Tag','mutation', ...
                    'ToolTipString','Choose appropriate mutation for your datatype.', ...                     
                    'UserData', mutation ); 
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','mutation', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               

yl = yl;
xl = xl + frdx*dx;
mutation_rate = 1;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx dy], ...
                    'String', int2str(mutation_rate), ...
                    'Style','edit', ...
                    'Tag','mutation_rate', ...
                    'ToolTipString','Mutation rate defines how many genes in each chromosome will be mutated (default is 1).', ...                     
                    'UserData', mutation_rate );
xl = xl - frdx*dx;

yl = yl - dy;
mutation_size = 1.0;
ih = ih + 1;
hui(ih)= uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx frdy*dy], ...
                    'String',num2str(mutation_size), ...
                    'Style','edit', ...
                    'Tag','mutation_size', ...
                    'ToolTipString','Mutation size is used for numeric datatypes and defines initial mutation step.', ...                     
                    'UserData', mutation_size);

xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx frdy*dy], ...
                   'String','size', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               

yl = yl;
xl = xl + frdx*dx;
mutation_decay_rate = 1.0;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx dy], ...
                    'String', num2str(mutation_decay_rate), ...
                    'Style','edit', ...
                    'Tag','mutation_decay_rate', ...
                    'ToolTipString','Mutation decay rate is used for real datatype and defines decrease rate of mutation step.', ...                     
                    'UserData', mutation_decay_rate );
xl = xl - frdx*dx;


% crossover
yl = yl - dy/frdy;                
crossover = {'onepoint','twopoint','uniform','arithmetic'};
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx dy], ...
                    'String',crossover, ...
                    'Style','popupmenu', ...
                    'Value', 1 , ...
                    'Tag','crossover', ...
                    'ToolTipString','Crossover type.', ...                                         
                    'UserData', crossover ); 
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx dy], ...
                   'String','crossover', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;               

yl = yl;
xl = xl + frdx*dx;
crossover_rate = 1;
ih = ih + 1;
hui(ih) = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl dx dy], ...
                    'String', int2str(crossover_rate), ...
                    'Style','edit', ...
                    'Tag','crossover_rate', ...
                    'ToolTipString','Crossover rate (0.0-1.0).', ...                                         
                    'UserData', crossover_rate );
xl = xl - frdx*dx;

% fname
yl = yl - dy/frdy;                
huifn = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl frdx*dx frdy*dy], ...
                    'String','', ...
                    'Style','edit', ...
                    'Tag','fname', ...
                    'ToolTipString','Enter the function name', ...                                         
                    'UserData', crossover ); 
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx frdy*dy], ...
                   'String','fun.name', ...
                   'Style','text', ...
                   'Value',1);
xl = xl + frdx*dx;

% additional varliables
yl = yl - 2*frdy*dy;                
huifn = uicontrol('Parent',h0, ...
                    'Units','normalized', ...
                    'Position',[xl yl 2*frdx*dx 2*frdy*dy], ...
                    'String','', ...
                    'Style','edit', ...
                    'Tag','vnames', ...
                    'ToolTipString','Enter names of additional variables, separated by comas (example: ,P0,P1,P2). Variables with these names must exist in the Matlab workspace.', ...                                         
                    'Max', 2 ); 
xl = xl - frdx*dx;
hdtype = uicontrol('Parent',h0, ...
                   'Units','normalized', ...
                   'Position',[xl yl frdx*dx frdy*dy], ...
                   'String','add.variables', ...
                   'Style','text', ...
                   'Value',1);               

% Define tags that will be searched for configuration               
tags = cell(ih,1);
for i = 1:ih
    tags{i} = get(hui(i),'Tag');
end
set(h0,'UserData',tags);

% Run button
yl = yl - dy;                
uicontrol('Parent',h0, ...
          'Units' ,'normalized', ...
          'Position', [xl yl frdx*dx frdy*dy] , ... 
          'String','Run', ...
          'Callback', 'rcod = 1; runcback');

% Save button
xl = xl + frdx*dx;                
uicontrol('Parent',h0, ...
          'Units' ,'normalized', ...
          'Position', [xl yl frdx*dx frdy*dy] , ... 
          'String','Save', ...
          'Callback', 'rcod = 2; runcback');

% Load button
xl = xl + frdx*dx;                
uicontrol('Parent',h0, ...
          'Units' ,'normalized', ...
          'Position', [xl yl frdx*dx frdy*dy] , ... 
          'String','Load', ...
          'Callback', 'rcod = 3; runcback');
end

% load configuration data and show it on the figure window
if ( nargin >= 1 )
    tags = get(h0,'UserData');

    for i = 1:size(config_i,1)
        ho = findobj(h0, 'Tag', config_i{i,1});
        if ( prod(size(ho)) == 1 ) 
           if ( strcmp(get(ho,'Style'),'popupmenu') )
               ud = get(ho,'UserData');
               strcfg = config_i{i,2};
               for j = 1:prod(size(ud))
                  if( strcmp(ud{j},config_i{i,2}) ) 
                     set(ho,'Value',j);
                     break;
                  end
               end
           else
               set(ho,'String',config_i{i,2});
           end
       end
       if ( nargin >= 2 )
          set(findobj(h0, 'Tag','fname'),'String' ,fname_i );
       end
       if ( nargin >= 3 )
          set(findobj(h0, 'Tag','vnames'),'String',vnames_i);    
       end
   end
end

% get configuration from the figure window      
if ( nargout >= 1 )
    h0 = gcf;
    tags = get(h0,'UserData');
    config = tags;
    for i = 1:size(tags,1)
        ho = findobj(h0, 'Tag', tags{i});
        if ( isempty(ho) | prod(size(ho)) > 1 ) 
            error('Unknown confguration option')
        end
        if ( strcmp(get(ho,'Style'),'popupmenu') )
            ud = get(ho,'UserData');
            in = get(ho,'Value');
            config{i,2} = ud{in};
        else
            config{i,2} = get(ho,'String');
        end
    end
    % additional configuration data - not covered with the data from figure
    i = i+1; config{i,1} = 'first_generation'; config{i,2} = '0'; 
    i = i+1; config{i,1} = 'screen_output';    config{i,2} = 'off'; 
    i = i+1; config{i,1} = 'mutation_rate_constrained', config{i,2} = '1.0';
    if ( nargout >= 2 )
       fname  = get(findobj(h0, 'Tag','fname'),'String');
    end
    if ( nargout >= 3 )
       vnames = get(findobj(h0, 'Tag','vnames'),'String');    
    end
end 
