function epalghlp

section = get(gco,'Tag');
h0      = findobj(0,'Tag','guiepalg');
hhlp    = findobj(h0,'Tag','help');
hlpstr  = section;

switch section
   case 'guiepalg'
      hlpstr = {'Evolutionary Programming ALGorithm (EPALG)', ...
                '', ...
                'This is Matlab graphical user interface to EPALG.', ... 
                'If you want to see explanation for parameters, click with the right mouse button on the parameter field.'};
   
   case 'datatype'
      hlpstr = {'datatype', ...
                '', ...
                'It is most important parameter. There are 3 options:', ...
                ' 1. bit  - chromosome will be a bit string', ...
                ' 2. int  - chromosome will be a vector of unsigned integers', ...
                '           User can specify the size of integer in bits.', ...
                ' 3. real - chromosome will be a vector of floating point values.', ...
                '           Note that gene values are always inside [-1,1] limits.'};

   case 'integer_size'
      hlpstr = {'integer_size', ...
                '', ...
                'Specifies how many bits are used to represent each unsigned integer.', ...
                'Cannot exceed the size of a native int.'};
   
   case 'generations'
      hlpstr = {'generations', ...
                '', ...
                'Number of generations to run GA.'};
   
   case 'population'
      hlpstr = {'population', ...
                '', ...
                'Size of the population in terms of the number of chromosomes.'};
   
   case 'length'
      hlpstr = {'length', ...
                '', ...
                'Number of genes in the chromosome. If the datatype is bit, length is the number of bits in the chromosome.'};
   
   case 'seed'
      hlpstr = {'seed', ...
                '', ...
                'Used for initializing the random number generator.'};
   
   case 'selection'
      hlpstr = {'selection', ...
                '', ...
                'There are 2 different selection options:', ... 
                '1. roulette - chance of selection is proportional to the fitness', ...
                '3. tournament - several chromosomes compete for selection', ...
                '               (the number of competitors are defined by tornament size)'};
   
   case 'selection_tournament_size'
      hlpstr = {'selection_tournament_size', ...
                '', ...
                'Defines the number of chromosomes competing for selection', ...
                '(if selection is not tournament, parameter has no effect)'};
   
   case 'normalisation' 
      hlpstr = {'normalisation', ...
                '', ...
                'Defines the way for calculating fitness from raw function values.', ...
                'Following options are avaible:', ...
                '1. direct  - calculate fitness directly from function values', ...
                '2. reverse - the same as direct except that it tries to find minimum', ...
                '3. rank_linear - calculate the fitness based on how solutions are ranked', ...
                '                 (eg. worse solution has lowest rank: 0, 1, 2, ...)     ', ...
                '4. reverse_rank_linear - is the same as rank linear, except it tries to find min.' };
   
   case 'replacement' 
      hlpstr = {'replacement', ...
                '', ...
                'Defines how new chromosomes (coded solutions) will replace old ones:', ...
                '1. uniform - solutions to be replaced are picked at random', ...
                '2. ranked  - solutions to be replaced are picked based on the rank'};
   
   case 'replacement_condition'
      hlpstr = {'replacement_condition', ...
                '', ...
                'Defines the terms of replacement:', ... 
                '1. unconditional - selected old solutions are always replaced by new ones',...
                '2. if_improved - old solutions that have better fitness values are not replaced'};
   
   case 'replacement_rate'
      hlpstr = {'replacement_rate', ...
                '', ...
                'Defines the portion of the population that will be replaced in each generation.', ... 
                'Valid values: 0.0 - 1.0'};
   
   case 'elitism'
      hlpstr = {'elitism', ...
                '', ...
                'If elitism is on, it means that the best solution will not be replaced regardles,', ...
                'the specified replacement options'};         
   
   case 'mutation'
      hlpstr = {'mutation', ...
                '', ...
                'There are several mutation options related to the selected datatype:', ...
                '1. invert   - (datatype bit only) flips some bits to the opposite value', ...
                '2. reinit   -  reinitialises some genes (all datatypes)', ...
                '3. step     - (numeric datatypes) adds or subtracts a fixed amount to/from some genes', ...
                '4. uniform  - (numeric datatypes) adds a uniformly distributed random variable (with mean 0)', ...
                '5. gaussian - (numeric datatypes) adds a normally distributed random variable (mean 0)'};

   case 'mutation_rate'
      hlpstr = {'mutation_rate', ...
                '', ...
                'Controls how many genes in each chromosome are mutated.', ...
                'Valid values: from 0 to length of chromosome.', ...
                'Default value is 1 which means that on average 1 gene will be mutated per chromosome.'};
 
   case 'mutation_size'
      hlpstr = {'mutation_size', ...
                '', ... 
                'Controls the size of mutations for the numeric datatypes:', ...
                'Specifies the size of changes for step mutation.', ...
                'Specifies the range of changes for uniform mutation (from -size to +size).', ...
                'Specifies the standard deviation of Gaussian mutation.'};

   case 'mutation_decay_rate'
      hlpstr = {'mutation_decay rate', ...
                '', ... 
                ' Step, uniform and gaussian mutation can be forced to decrease', ...
                ' in size as the algorithm progresses; this helps to fine-tune', ...
                ' changes during convergence.The mutation size is multiplied by the', ... 
                ' mutation_decay_rate on each generation;', ...
                ' It should therefore be slightly less than 1.0 (e.g. 0.95-0.98).', ...
                ' With the default value of 1.0 no decay takes place.'};

   case 'crossover'
      hlpstr = {'crossover', ...
                '', ... 
                'Specifies the type of crossover used.', ...
                '1. onepoint - the two parents a joined at a randomly selected crossover point', ...
                '2. twopoint - the two parents are joined at two crossover points', ...
                '3. uniform  - children are produced by randomly selecting each gene from each parent', ... 
                '4. arithemtic - (for real datatypes only) children is made by calculating random point', ...
                '                 between two parents: (c1 = r*p1+(1-r)*p2)'};
   
   case 'crossover_rate'
      hlpstr = {'crossover_rate', ...
                '', ... 
                'Specifies the probability off crossover. Valid values are from 0.0 to 1.0.'};
   
   case 'fname'
      hlpstr = {'fname', ...
                '', ... 
                'Enter the name of the function that will be called for evaluation.'};
   
   case 'vnames'
      hlpstr = {'vnames', ...
                '', ... 
                'User can specify additional variables that will be passed to evaluation function.', ... 
                'The list should start with comma and maximum number of vairables is 15 (example: ,v0,v1,v2).',...
                'Note that listed variables must exit in Matlab workspace or the call to a function (fname) will issue an error message.'};
end

if ( ~isempty(hlpstr) )
   set(hhlp,'String',hlpstr)
end
